/*
 * Perform SPI communication using GPIO operations.
 *
 * Copyright (C) 2018, 2020, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/devices/gpio.h>
#include <stdint.h>

#pragma once



#ifdef __cplusplus

/* SPI peripheral device. */

class Spi_gpio
{
  Hw::Gpio_chip *_clock_device;
  int _clock_pin;
  Hw::Gpio_chip *_data_device;
  int _data_pin;
  Hw::Gpio_chip *_enable_device;
  int _enable_pin;
  uint32_t _frequency;

public:
  /* Associate the device with a particular memory region. */

  explicit Spi_gpio(Hw::Gpio_chip *clock_device, int clock_pin,
                    Hw::Gpio_chip *data_device, int data_pin,
                    Hw::Gpio_chip *enable_device, int enable_pin,
                    uint32_t frequency = 0);

  void send(int bytes, uint8_t data[]);
};

#endif /* __cplusplus */



/* C language interface. */

EXTERN_C_BEGIN

void *spi_gpio_get_channel(void *clock_chip, int clock_pin,
                           void *data_chip, int data_pin,
                           void *enable_chip, int enable_pin,
                           uint32_t frequency);

void spi_gpio_send(void *channel, int bytes, uint8_t data[]);

EXTERN_C_END
